from vcCommand import *
import os.path
import xml.etree.ElementTree as xml
 
app = getApplication()

def addProperty(ps,p):
  if p.Type == VC_DISTRIBUTION or p.Type == VC_BUTTON or p.Type == VC_MATERIAL or not p.IsVisible or not p.IsPersistent or not p.Value:
    return
  property = xml.Element('Property')
  ps.append(property)
  property.attrib['Name'] = p.Name  
  if p.Type == VC_VECTOR:
    property.attrib['Value.X'] = str(p.Value.X)
    property.attrib['Value.Y'] = str(p.Value.Y)
    property.attrib['Value.Z'] = str(p.Value.Z)
    property.attrib['Value.W'] = str(p.Value.W)
  elif p.Type == VC_MATRIX:
    property.attrib['Value.Nx'] = str(p.Value.N.X)
    property.attrib['Value.Ny'] = str(p.Value.N.Y)
    property.attrib['Value.Nz'] = str(p.Value.N.Z)
    property.attrib['Value.Nw'] = str(p.Value.N.W)
    property.attrib['Value.Ox'] = str(p.Value.O.X)
    property.attrib['Value.Oy'] = str(p.Value.O.Y)
    property.attrib['Value.Oz'] = str(p.Value.O.Z)
    property.attrib['Value.Ow'] = str(p.Value.O.W)
    property.attrib['Value.Ax'] = str(p.Value.A.X)
    property.attrib['Value.Ay'] = str(p.Value.A.Y)
    property.attrib['Value.Az'] = str(p.Value.A.Z)
    property.attrib['Value.Aw'] = str(p.Value.A.W)
    property.attrib['Value.Px'] = str(p.Value.P.X)
    property.attrib['Value.Py'] = str(p.Value.P.Y)
    property.attrib['Value.Pz'] = str(p.Value.P.Z)
    property.attrib['Value.Pw'] = str(p.Value.P.W)
  else:
    property.attrib['Value'] = str(p.Value)

def state1():
  save = app.findCommand("dialogSaveFile")
  uri = ""
  ok = True
  filter = "XML model file (*.xml)|*.xml"
  save.execute(uri,ok,filter,'Choose File to save model to')
  if not save.Param_2:
    print "No file selected, aborting command."
    return
  uri = save.Param_1
  filename = uri[8:len(uri)]
  root = xml.Element('Model')
  components = xml.Element('Components')
  root.append(components)
  connections = xml.Element('Connections')
  root.append(connections)
  attachments = xml.Element('Attachments')
  root.append(attachments)
  cmap = {}
  cc = 0
  for c in app.Components:
    cc += 1
    cmap[c] = cc
    if not c.VCID:
      print c.Name, ' has no VCID'
    else:
      component = xml.Element('Component')
      components.append(component)
      component.attrib['Name'] = c.Name
      component.attrib['GlobalIdentity'] = c.VCID    
      component.attrib['LocalIdentity'] = str(cmap[c])
      #component.attrib['Uri'] = c.Uri    # NOT WORKING WITHIN LAYOUTS ?
      location = xml.Element('Location')
      component.append(location)
      location.attrib['Nx'] = ('%f' % c.WorldPositionMatrix.N.X)
      location.attrib['Ny'] = ('%f' % c.WorldPositionMatrix.N.Y)
      location.attrib['Nz'] = ('%f' % c.WorldPositionMatrix.N.Z)
      location.attrib['Nw'] = ('%f' % c.WorldPositionMatrix.N.W)
      location.attrib['Ox'] = ('%f' % c.WorldPositionMatrix.O.X)
      location.attrib['Oy'] = ('%f' % c.WorldPositionMatrix.O.Y)
      location.attrib['Oz'] = ('%f' % c.WorldPositionMatrix.O.Z)
      location.attrib['Ow'] = ('%f' % c.WorldPositionMatrix.O.W)
      location.attrib['Ax'] = ('%f' % c.WorldPositionMatrix.A.X)
      location.attrib['Ay'] = ('%f' % c.WorldPositionMatrix.A.Y)
      location.attrib['Az'] = ('%f' % c.WorldPositionMatrix.A.Z)
      location.attrib['Aw'] = ('%f' % c.WorldPositionMatrix.A.W)
      location.attrib['Px'] = ('%f' % c.WorldPositionMatrix.P.X)
      location.attrib['Py'] = ('%f' % c.WorldPositionMatrix.P.Y)
      location.attrib['Pz'] = ('%f' % c.WorldPositionMatrix.P.Z)
      location.attrib['Pw'] = ('%f' % c.WorldPositionMatrix.P.W)
      properties = xml.Element('Properties')
      component.append(properties)
      for p in c.Properties:
        addProperty(properties,p)
  for c in app.Components:
    for iface in c.findBehavioursByType(VC_ONETOONEINTERFACE):
      if iface.ConnectedComponent:
        for cts in iface.ConnectedToSections:
          connection = xml.Element('Connection')
          connections.append(connection)
          connection.attrib['FromComponent'] = str(cmap[c])
          connection.attrib['ToComponent'] = str(cmap[cts.Interface.Component])
          connection.attrib['FromInterface'] = iface.Name
          connection.attrib['ToInterface'] = cts.Interface.Name
  for c in app.Components:
    if c.VCID and c.Parent.Name != 'ROOT':
      attachment = xml.Element('Attachment')
      attachments.append(attachment)
      attachment.attrib['ParentComponent'] = str(cmap[c.Parent.Component])
      attachment.attrib['ParentNodeName'] = c.Parent.Name
      attachment.attrib['ChildComponent'] = str(cmap[c])
  file = open(filename, 'w')
  xml_indent(root, 0)
  xml.ElementTree(root).write(file)
  file.close()
  print 'Done'

def xml_indent(elem, level=0):
  i = "\n" + level*"  "
  if len(elem):
    if not elem.text or not elem.text.strip():
      elem.text = i + "  "
    if not elem.tail or not elem.tail.strip():
      elem.tail = i
    for elem in elem:
      xml_indent(elem, level+1)
    if not elem.tail or not elem.tail.strip():
      elem.tail = i
  else:
    if level and (not elem.tail or not elem.tail.strip()):
      elem.tail = i


addState(state1)